#![feature(if_let_guard)]
use crate::game_handler::*;
use crate::outgoing_message_handler::*;
use axum::extract::ws::Message;
use lib::*;
use std::fmt::Debug;
use std::net::SocketAddr;
use tokio::sync::mpsc;
use user_handler::*;
use uuid::Uuid;
pub mod card_loader;
pub mod game;
pub mod game_handler;
pub mod incoming_message_handler;
pub mod name_generator;
pub mod outgoing_message_handler;
pub mod user_handler;
pub mod websocket;

/// User
#[derive(Debug)]
pub struct User {
    pub uuid: String,
    pub name: String,
    pub tx: mpsc::Sender<Message>,
}

impl User {
    /// Create a new user object from incoming data
    pub fn new(name: String, tx: mpsc::Sender<Message>) -> User {
        User {
            name,
            tx,
            uuid: Uuid::now_v7().to_string(),
        }
    }

    pub fn change_name(&mut self, new_name: String) {
        self.name = new_name;
    }
}

/// Shared state
pub struct AppState {
    pub tx_broadcast: tokio::sync::broadcast::Sender<Message>,
    pub tx_game_handler: mpsc::Sender<GameHandlerMessage>,
    pub tx_incoming_message_handler: mpsc::Sender<(SocketAddr, Message)>,
    pub tx_outgoing_message_handler:
        mpsc::Sender<OutgoingMessageHandlerMessage<ServerToClientMessage>>,
    pub tx_user_handler: mpsc::Sender<UserHandlerMessage>,
}
