use anyhow::{Context, Result};
use rand::prelude::IndexedRandom;
use std::{
    collections::HashSet,
    fs::File,
    io::{BufRead, BufReader},
};

pub struct NameGenerator {
    first_names: Vec<String>,
    last_names: Vec<String>,
}

/// Parse name list
pub fn load_names(path: &str) -> Result<Vec<String>> {
    let f = File::open(path).with_context(|| format!("Invalid names path: \"{}\"", path))?;
    let f = BufReader::new(f);

    let mut buf = vec![];

    for line in f.lines() {
        buf.push(line?)
    }

    Ok(buf)
}

impl NameGenerator {
    pub fn new() -> Self {
        NameGenerator {
            first_names: load_names("data/first.txt").unwrap(),
            last_names: load_names("data/last.txt").unwrap(),
        }
    }

    pub fn generate(&mut self, reserved_names: &HashSet<String>) -> String {
        // Roll for username and re-roll if taken
        let mut name;

        loop {
            name = format!(
                "{} {}",
                self.first_names.choose(&mut rand::rng()).unwrap(),
                self.last_names.choose(&mut rand::rng()).unwrap(),
            );

            if !reserved_names.contains(&name) {
                break;
            }
        }

        name
    }
}
