use crate::OutgoingMessageHandlerMessage::*;
use axum::extract::ws::Message;
use lib::*;
use serde::Serialize;
use serde_json::to_string;

/// For interacting with the outgoing message handler
pub enum OutgoingMessageHandlerMessage<T: Serialize> {
    Unicast((tokio::sync::mpsc::Sender<Message>, T)),
    Broadcast(T),
}

pub struct OutgoingMessageHandler {
    pub tx_broadcast: tokio::sync::broadcast::Sender<Message>,
}

impl OutgoingMessageHandler {
    pub fn new(tx_broadcast: tokio::sync::broadcast::Sender<Message>) -> Self {
        OutgoingMessageHandler { tx_broadcast }
    }

    pub async fn handle(&self, message: OutgoingMessageHandlerMessage<ServerToClientMessage>) {
        match message {
            Unicast(message) => {
                if let Err(e) = message.0.send(self.serialize(message.1)).await {
                    tracing::error!("Error sending message: {}", e)
                }
            }
            Broadcast(message) => {
                if let Err(e) = self.tx_broadcast.send(self.serialize(message)) {
                    tracing::error!("Error sending message: {}", e)
                }
            }
        }
    }

    fn serialize<T: Serialize>(&self, message: T) -> Message {
        let msg = Message::Text(to_string::<T>(&message).unwrap().into());
        return msg;
    }
    pub fn to_user<T: Serialize>(_channel: tokio::sync::mpsc::Sender<Message>, _message: T) {
        todo!()
    }

    pub fn to_all<T: Serialize>(_channel: tokio::sync::broadcast::Sender<Message>, _message: T) {
        todo!()
    }
}
